#include <boost/iterator.hpp>

#include <boost/python.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/manage_new_object.hpp>

#include <boost/python/register_ptr_to_python.hpp>
using namespace boost::python;

#include "alg/seq_span.hpp"

#include "stl_container_adapter.hpp"

SeqSpanRef seq_span_factory(
  const std::string& s, 
  AlphabetRef a = dna_alphabet, 
  SeqSpan::strand_type strand = SeqSpan::PlusStrand)
{
  SeqSpanRef seq(new SeqSpan(s, a, strand));
  return seq;
}

BOOST_PYTHON_FUNCTION_OVERLOADS(
  seq_span_factory_overloads, 
  seq_span_factory,
  1, // minimum arguments
  3 // maximum arguments
);

void export_seq_span()
{
  enum_<SeqSpan::strand_type>("strand")
    .value("unknown", SeqSpan::UnknownStrand)
    .value("minus", SeqSpan::MinusStrand)
    .value("plus", SeqSpan::PlusStrand)
    .value("both", SeqSpan::BothStrand)
    .value("same", SeqSpan::SameStrand)
    .value("opposite", SeqSpan::OppositeStrand)
    .value("single", SeqSpan::SingleStrand)
    ;

  def("SeqSpan", seq_span_factory, 
      seq_span_factory_overloads(args("sequence", "alphabet", "strand"))
      );

  class_<SeqSpan>("_SeqSpan", no_init)
    .def("__getitem__", &std_item<SeqSpan>::get_const,
        return_value_policy<copy_const_reference>())
    .def("__len__", &SeqSpan::size)
    .def("__str__", &SeqSpan::sequence)
    .def("empty", &SeqSpan::empty)
    //.add_property("alphabet", 
    //              make_getter(&SeqSpan::get_alphabet,
    //                          return_value_policy<copy_const_reference>()))
    .add_property("annotations", &SeqSpan::annotations, 
                                 &SeqSpan::setAnnotations)
    .add_property("drawable", &SeqSpan::drawable, &SeqSpan::setDrawable)
    .add_property("start", &SeqSpan::start, &SeqSpan::setStart,
                  "start position relative to root sequence")
    .add_property("stop", &SeqSpan::stop, &SeqSpan::setStop,
                  "one past the last position relative to the root sequence.")
    .add_property("strand", &SeqSpan::strand, 
                  "describe which strand the span is on")

    .add_property("parentStart", &SeqSpan::parentStart, &SeqSpan::setParentStart,
                  "start position relative to parent sequence")
    .add_property("parentStop", &SeqSpan::parentStop, &SeqSpan::setParentStop,
                  "one past the last position relative to the parent sequence.")
    // doesn't work because I keep getting new python objects for the same
    // shared ptr
    //.def("parent", &SeqSpan::parent)
    .def("subseq", &SeqSpan::subseq)
    ;
 
  register_ptr_to_python< SeqSpanRef >();
}
