import os
import sys
import unittest

# kinda hackish but it makes it possible to runi under ctest 
sys.path.append(os.getcwd())

import mussa

class TestSeqSpan(unittest.TestCase):
  def testSimple(self):
    seq_text = "AGCT"
    s = mussa.SeqSpan(seq_text)
    self.failUnlessEqual(str(s), seq_text)

  def testIter(self):
    seq_text = "AAAAAAGGGGGGG"
    s = mussa.SeqSpan(seq_text)
    new_seq_text = "".join( [ x for x in s ] )
    print "len", len(s)
    self.failUnlessEqual(len(seq_text), len(s))
    self.failUnlessEqual(seq_text, new_seq_text)

  def testSettingStrand(self):
    seq_text = "AAAAG"
    complement_text = "CTTTT"
    splus = mussa.SeqSpan(seq_text, mussa.alphabet.dna_alphabet, mussa.strand.plus)
    sminus = splus.subseq(0, len(seq_text), mussa.strand.minus)
    self.failUnlessEqual(str(sminus), complement_text)

  # Doesn't work because I keep getting new python objects created
  # for calls to SeqSpan::parent
  #def testParentEquality(self):
  #  seq_text = "AAAAAAGGG"
  #  sparent = mussa.SeqSpan(seq_text)
  #  s1 = sparent.subseq(4,4, mussa.strand.minus)
  #y  s2 = s1.subseq(0,2, mussa.strand.opposite)
  #  print "s2 parent call 1",s2.parent()
  #  print "s2 parent call 2", s2.parent()
  #  self.failUnless(s1 is s2.parent())
  #  self.failUnlessEqual(s1, s2.parent())
  #  self.failUnless(sparent is s1.parent())
  #  self.failUnlessEqual(sparent == s1.parent())

  def testAnnotations(self):
    seq_text = "AAAAAG"
    name = "As"
    s = mussa.SeqSpan(seq_text)
    s.annotations = mussa.Annotations(name)
    self.failUnlessEqual(s.annotations.name, name)
    self.failUnlessEqual(s.annotations['name'], name)

    term_key = 'GO term'
    term = "term"
    s.annotations[term_key] = term
    self.failUnlessEqual(s.annotations[term_key], term)

  def testDrawable(self):
    # ok this is a boring test...
    seq_text = "AAAAAG"
    s = mussa.SeqSpan(seq_text)
    d = mussa.Drawable()
    s.drawable = d

    self.failUnless(s.drawable is d)


def suite():
  return unittest.makeSuite(TestSeqSpan, 'test')

if __name__ == "__main__":
  sys.exit(unittest.main(defaultTest='suite'))
